package crypto;

import java.security.Key;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;

import exception.InternalEncryptionException;

// TODO: Auto-generated Javadoc
/**
 * The Class CustomCrypto.
 */
public class CustomCrypto {
	/**
	 * Cuanto mayor sea el nmero de iteraciones mayor complejidad computacional
	 * y seguridad ante ataques.
	 */
	private static final int iterations = 2 * 100;
	
	/** The Constant saltLen. */
	private static final int saltLen = 32;
	
	/** The Constant desiredKeyLen. */
	private static final int desiredKeyLen = 256;

	/** The Constant ALGO. */
	private static final String ALGO = "AES";

	/**
	 * Calcula un Salt PBKDF2 sobre la contrasea dada en texto plano para
	 * obtener un valor cifrado almacenable en base de datos.
	 *
	 * @param password the password
	 * @return the salted hash
	 * @throws Exception the exception
	 */
	public static String getSaltedHash(String password) throws Exception {
		byte[] salt = SecureRandom.getInstance("SHA1PRNG")
				.generateSeed(saltLen);
		// store the salt with the password
		return Base64.encodeBase64String(salt) + "$" + hash(password, salt);
	}

	/**
	 * Valida si una contrasea dada en texto plano corresponde con la
	 * contrasea cifrada y almacenada en base de datos.
	 *
	 * @param password the password
	 * @param stored the stored
	 * @return true, if successful
	 * @throws InternalEncryptionException the internal encryption exception
	 */
	public static boolean check(String password, String stored)
			throws InternalEncryptionException {
		String[] saltAndPass = stored.split("\\$");
		if (saltAndPass.length != 2) {
			throw new InternalEncryptionException(
					"Contrasea con formato no vlido");
		}
		String hashOfInput = hash(password, Base64.decodeBase64(saltAndPass[0]));
		return hashOfInput.equals(saltAndPass[1]);
	}

	/**
	 * Mtod privado que clacula una hash partiendo la contrasea en texto plano
	 * y un salt para mejorar la seguridad.
	 *
	 * @param password the password
	 * @param salt the salt
	 * @return the string
	 * @throws InternalEncryptionException the internal encryption exception
	 */
	private static String hash(String password, byte[] salt)
			throws InternalEncryptionException {
		if (password == null || password.length() == 0)
			throw new InternalEncryptionException(
					"No se soportan contraseas sin contenido.");
		SecretKeyFactory f;
		try {
			f = SecretKeyFactory.getInstance("PBKDF2WithHmacSHA1");
			SecretKey key = f.generateSecret(new PBEKeySpec(password
					.toCharArray(), salt, iterations, desiredKeyLen));
			return Base64.encodeBase64String(key.getEncoded());
		} catch (InvalidKeySpecException | NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			throw new InternalEncryptionException(e.getMessage());
		}
	}

	/**
	 * Cifra los datos en base a una contrasea maestra.
	 *
	 * @param masterKey the master key
	 * @param Data the data
	 * @return the string
	 * @throws InternalEncryptionException the internal encryption exception
	 */
	public static String encrypt(String masterKey, String Data)
			throws InternalEncryptionException {

		if (masterKey == null || Data == null)
			return null;

		Key key;
		try {
			key = generateKey(masterKey);
			Cipher c = Cipher.getInstance(ALGO);
			c.init(Cipher.ENCRYPT_MODE, key);
			byte[] encVal = c.doFinal(Data.getBytes());
			String encryptedValue = Base64.encodeBase64String(encVal);

			return encryptedValue;

		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw new InternalEncryptionException(e.getMessage());
		}
	}

	/**
	 * Descifra los datos en base a una contrasea maestra.
	 *
	 * @param masterKey the master key
	 * @param encryptedData the encrypted data
	 * @return the string
	 * @throws Exception the exception
	 */
	public static String decrypt(String masterKey, String encryptedData)
			throws Exception {

		if (masterKey == null || encryptedData == null)
			return null;

		Key key = generateKey(masterKey);
		Cipher c = Cipher.getInstance(ALGO);
		c.init(Cipher.DECRYPT_MODE, key);
		byte[] decordedValue = Base64.decodeBase64(encryptedData);
		byte[] decValue = c.doFinal(decordedValue);
		String decryptedValue = new String(decValue);
		return decryptedValue;
	}

	/**
	 * Generate key.
	 *
	 * @param masterKey the master key
	 * @return the key
	 * @throws Exception the exception
	 */
	private static Key generateKey(String masterKey) throws Exception {
		MessageDigest digest = MessageDigest.getInstance("MD5");
		byte[] hashedBytes = digest.digest(masterKey.getBytes("UTF-8"));

		Key key = new SecretKeySpec(hashedBytes, ALGO);
		return key;
	}

	/**
	 * The main method.
	 *
	 * @param args the arguments
	 * @throws Exception the exception
	 */
	public static void main(String[] args) throws Exception {

		String masterPassword = "1";

		String password = "1231231";
		String passwordEnc = encrypt(masterPassword, password);
		String passwordDec = decrypt(masterPassword, passwordEnc);

		System.out.println("Plain Text : " + password);
		System.out.println("Encrypted Text : " + passwordEnc);
		System.out.println("Decrypted Text : " + passwordDec);
	}
}