/** 
 * @author ngel Arias Romero. 
 * UOC - TFC SaveMeSafe 2015
 */
package ejb;

import java.util.List;
import java.util.UUID;

import javax.annotation.Resource;
import javax.ejb.Asynchronous;
import javax.ejb.Stateless;
import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.ws.rs.core.MediaType;

import jpa.UserJPA;

import org.jboss.logging.Logger;

import crypto.CustomCrypto;
import exception.CannotSendMailException;
import exception.CustomWrapperException;
import exception.InternalEncryptionException;
import exception.InternalPersistException;
import exception.UserAccessException;
import exception.UserNotFoundException;

// TODO: Auto-generated Javadoc
/**
 * EJB Session Bean Class.
 */
@Stateless
public class UserFacadeBean implements UserFacadeRemote {

	/** The entman. */
	@PersistenceContext(unitName = "SaveMeSafe")
	private EntityManager entman;

	/** The Constant LOGGER. */
	private static final Logger LOGGER = Logger.getLogger(UserFacadeBean.class);

	/** The gmail session. */
	@Resource(mappedName = "java:/mail/gmail")
	Session gmailSession;

	// @Inject
	// private Logger log;

	/**
	 * Login user.
	 *
	 * @param userName            , userPassword User to add.
	 * @param userPass            the user pass
	 * @return the new user instance
	 * @throws CustomWrapperException the custom wrapper exception
	 */
	@Override
	public UserJPA doLogin(String userName, String userPass)
			throws CustomWrapperException {

		if (userName == null || userPass == null)
			return null;

		UserJPA user = null;

		try {

			@SuppressWarnings("unchecked")
			List<UserJPA> listUserData = entman
					.createQuery("FROM UserJPA b WHERE b.name = :name")
					.setParameter("name", userName).getResultList();
			if (listUserData.size() == 1) {
				// Existe el usuario.
				user = listUserData.get(0);

				if (user.isBlocked()) {
					if (user.getTryNumber() == 0) {
						LOGGER.error("El usuario "
								+ userName
								+ " ha intentado acceder a la aplicacin sin realizar la activacin previamente");
						throw new UserAccessException("Usuario no activado");
					} else {
						LOGGER.error("El usuario "
								+ userName
								+ " ha intentado acceder a la aplicacin pero tiene al acceso bloqueado");
						throw new UserAccessException("Usuario bloqueado");
					}
				}

				// Validar contrasea codificada
				try {
					if (CustomCrypto.check(userPass, user.getPassword())) {
						user.setTryNumber(0);
						LOGGER.info("El usuario " + userName
								+ " ha hecho login en la aplicacin");
						return user;
					} else {
						user.setTryNumber(user.getTryNumber() + 1);
						if (user.getTryNumber() >= 3) {
							user.setBlocked(true);
							LOGGER.error("El usuario "
									+ userName
									+ " ha intentado acceder a la aplicacin pero ha agotado el nmero de intentos");
							throw new UserAccessException(
									"La cuenta de usuario ha sido bloqueada");
						}
						throw new UserAccessException(
								"Contrasea incorrecta para el usuario "
										+ userName);
					}
				} catch (InternalEncryptionException e) {
					// TODO Auto-generated catch block
					throw new UserAccessException(e.getMessage());
				}

			} else {
				LOGGER.error("El usuario "
						+ userName
						+ " ha intentado acceder a la aplicacin pero no est registrado");
				throw new UserNotFoundException("Usuario " + userName
						+ " no encontrado");
			}

		} catch (PersistenceException e) {
			throw new InternalPersistException(e.getMessage());
		}
	}

	/**
	 * Adds a new user.
	 *
	 * @param newUser            the new user
	 * @return the user persisted instance
	 * @throws CustomWrapperException the custom wrapper exception
	 */
	@Override
	public UserJPA addUser(UserJPA newUser) throws CustomWrapperException {
		try {

			// Save password hashed from plain text.
			newUser.setPassword(CustomCrypto.getSaltedHash(newUser
					.getPlainPassword()));

			// Create a user actionToken which is needed for activation
			newUser.setActionToken(UUID.randomUUID().toString());

			// User needs activate access account before login.
			newUser.setTryNumber(0);
			newUser.setBlocked(true);

			entman.persist(newUser);

			return newUser;

		} catch (Exception e) {
			throw new InternalPersistException(
					"addUser: Error registering user " + newUser.getName());
		}
		// log.info("Activated user with username {}.",
		// foundUser.getUsername());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ejb.UserFacadeRemote#updateUser(jpa.UserJPA)
	 */
	@Override
	public UserJPA updateUser(UserJPA currentUser) throws Exception {

		if (currentUser.getPlainPassword() != null)
			currentUser.setPassword(CustomCrypto.getSaltedHash(currentUser
					.getPlainPassword()));
		currentUser.setBlocked(false);
		currentUser.setTryNumber(0);
		UserJPA persistedUser = entman.find(UserJPA.class, currentUser.getId());
		if (persistedUser != null) {
			return entman.merge(currentUser);
		} else {
			UserJPA merged = entman.merge(currentUser);
			entman.persist(merged);
			entman.flush();
			return merged;
		}
	}

	/**
	 * Reads the user identified by the provided action token.
	 * 
	 * @param actionToken
	 *            the action token identifying the user
	 * @return the user
	 * @throws UserNotFoundException
	 *             if no user could be found
	 */
	@Override
	public UserJPA getUserByActionToken(String actionToken)
			throws UserNotFoundException {
		List<UserJPA> matchingUsers = entman
				.createQuery(
						"SELECT u FROM UserJPA u WHERE u.actionToken = :actionToken",
						UserJPA.class).setParameter("actionToken", actionToken)
				.getResultList();

		if (matchingUsers.size() != 1) {
			throw new UserNotFoundException(matchingUsers.size()
					+ " users matching given actionToken " + actionToken);
		}

		return matchingUsers.get(0);
	}

	/**
	 * Activates an existing user.
	 *
	 * @param user
	 *            the user to activate
	 * @return the user jpa
	 * @throws UserNotFoundException
	 *             if the user could not be found
	 */
	@Override
	public UserJPA activateUser(UserJPA user) throws UserNotFoundException {
		UserJPA foundUser = entman.find(UserJPA.class, user.getId());
		if (foundUser == null) {
			// log.error("Error activating user. User {} not found.",
			// user.getUsername());
			throw new UserNotFoundException("Error activating user. User "
					+ user.getName() + "not found.");
		}
		foundUser.setTryNumber(0);
		foundUser.setBlocked(false);
		foundUser.setActionToken(null);
		entman.persist(foundUser);
		return foundUser;
		// log.info("Activated user with username {}.",
		// foundUser.getUsername());
	}

	/**
	 * Reads the user identified by the provided identifier.
	 * 
	 * @param userIdentifier
	 *            the identifier identifying the user
	 * @return the user
	 * @throws UserNotFoundException
	 *             if no user could be found
	 */
	@Override
	public UserJPA getUserByIdentifier(String userIdentifier)
			throws UserNotFoundException {
		List<UserJPA> matchingUsers = entman
				.createQuery(
						"SELECT u FROM UserJPA u WHERE u.name = :userIdentifier OR u.email = :userIdentifier",
						UserJPA.class)
				.setParameter("userIdentifier", userIdentifier).getResultList();

		if (matchingUsers.size() != 1) {
			// log.error("{} users matching given identifier {}",
			// matchingUsers.size(), userIdentifier);
			throw new UserNotFoundException(matchingUsers.size()
					+ " users matching given identifier " + userIdentifier);
		}

		return matchingUsers.get(0);
	}

	/**
	 * Send email.
	 *
	 * @param email
	 *            the email
	 * @throws Exception
	 *             the exception
	 */
	@Asynchronous
	private void sendEmail(Email email) throws Exception {
		try {
			MimeMessage msg = new MimeMessage(gmailSession);
			msg.setHeader("Content-Type", "text/html;charset=UTF-8");
			msg.setSubject(email.getSubject(), "UTF-8");
			msg.setContent(email.getBody(), MediaType.TEXT_PLAIN);

			Address[] internetAdress;
			if (email.getAddress() != null) {
				internetAdress = new Address[email.getAddress().size()];
				for (int i = 0; i < email.getAddress().size(); i++) {
					internetAdress[i] = new InternetAddress(email.getAddress()
							.get(i));
				}
				msg.setRecipients(Message.RecipientType.TO, internetAdress);
			}

			Transport.send(msg);
		} catch (AddressException e) {
			throw e;
		} catch (MessagingException e) {
			throw e;
		}
	}

	/**
	 * Method responsible for sending a email to the user, including a link to
	 * activate his user account.
	 * 
	 * @param user
	 *            User the activation mail should be sent to
	 * @param serverString
	 *            Name and port of the server the user was added.
	 * @throws CannotSendMailException
	 *             if the mail could not be sent
	 */
	@Override
	public void sendActivationMail(UserJPA user, String serverString)
			throws CannotSendMailException {

		try {

			StringBuilder builder = new StringBuilder();
			builder.append("Hola " + user.getName() + ", \n\n");
			builder.append("Por favor, utiliza el siguiente enlace para confirmar tu cuenta de usuario en SameMeSafe: \n");
			builder.append("http://" + serverString
					+ "/savemesafe/public/activateUser.xhtml?uid="
					+ user.getActionToken());
			builder.append("\n\n--\n");
			builder.append("Reciba un cordial saludo del equipo de SaveMeSafe");

			sendEmail(new Email("Activacin de acceso", builder.toString(),
					user.getEmail()));

			// log.debug("Activation mail sent successfully to {}",
			// user.getEmail());
		} catch (Exception e) {
			// log.error("Error at sending activation mail to {}",
			// user.getEmail());
			throw new CannotSendMailException("Error enviando correo a  "
					+ user.getEmail(), e);
		}
	}

	/**
	 * Resets the password of the user identified by the actionToken.
	 *
	 * @param user
	 *            the user
	 * @return the user jpa
	 * @throws UserNotFoundException
	 *             if the user could not be found
	 */
	@Override
	public UserJPA resetPassword(UserJPA user) throws UserNotFoundException {

		// We have to find the user because if we use em.merge(user)
		// user.plainPassword will be deleted (because it is transient).
		UserJPA foundUser = entman.find(UserJPA.class, user.getId());
		if (foundUser == null) {
			// log.error("Error resetting password. User not found {}.",
			// user.getName());
			throw new UserNotFoundException(
					"Error resetting password. User not found "
							+ user.getName());
		}

		try {
			foundUser.setPlainPassword(user.getPlainPassword());
			foundUser.setPassword(CustomCrypto.getSaltedHash(user
					.getPlainPassword()));
			foundUser.setActionToken(null);
			foundUser.setTryNumber(0);
			foundUser.setBlocked(false);
			// foundUser.setStatus(IdpUserState.ACTIVE);
			entman.persist(foundUser);
			return foundUser;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// log.info("Reset password for user " + user.getName());
		return foundUser;
	}

	/**
	 * Initiates password reset for the user.
	 *
	 * @param user
	 *            the user
	 * @return the user jpa
	 */
	@Override
	public UserJPA initiateResetPassword(UserJPA user) {
		user.setBlocked(true);
		user.setActionToken(UUID.randomUUID().toString());
		entman.persist(entman.merge(user));
		return user;
		// log.info("Set action token for password reset mail for user {}",
		// user.getUsername());
	}

	/**
	 * Sends the user a link to reset the password.
	 * 
	 * @param user
	 *            the user
	 * @param serverString
	 *            host and port of the server
	 * @throws CannotSendMailException
	 *             if the password reset mail could not be sent
	 */
	@Override
	public void sendPasswordResetMail(UserJPA user, String serverString)
			throws CannotSendMailException {
		try {

			StringBuilder builder = new StringBuilder();
			builder.append("Hola " + user.getName() + ", \n\n");
			builder.append("Has solicitado recuperar la contrasea de acceso a SaveMeSafe");
			builder.append("\n\n");
			builder.append("Por favor haga click en el siguiente enlace para poder establecer una nueva contrasea de acceso: \n");
			builder.append("http://" + serverString
					+ "/savemesafe/public/resetPassword.xhtml?uid="
					+ user.getActionToken());
			builder.append("\n\n--\n");
			builder.append("Reciba un cordiar saludo del equipo SaveMeSafe");

			sendEmail(new Email("Reset de la Contrasea", builder.toString(),
					user.getEmail()));
			// log.debug("Sent password reset mail to " + user.getEmail());

		} catch (Exception e) {
			// log.error("Error at sending password reset mail to {}",
			throw new CannotSendMailException(
					"Error al enviar el correo a la cuenta " + user.getEmail());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ejb.UserFacadeRemote#deleteUser(jpa.UserJPA)
	 */
	@Override
	public Boolean deleteUser(UserJPA currentUser) throws Exception {
		// TODO Auto-generated method stub
		// entman.remove(currentUser);
		UserJPA persistedUser = entman.find(UserJPA.class, currentUser.getId());
		if (persistedUser != null) {
			entman.remove(persistedUser);
		} else {
			UserJPA merged = entman.merge(currentUser);
			entman.remove(merged);
			entman.flush();
		}
		return true;
	}
}
